#!/usr/bin/perl

# Usage:
# $ perl create_mammal_fly_b1h_sstat_net.pl --conf net_fly_motif_v2_B1H_mammals_v3_1.cfg > net_fly_motif_v2_B1H_mammals_v3_1.xgmml

use warnings;

use FindBin qw($Bin);
use lib "$Bin";

use AppConfig ':argcount';
use AppConfig::Getopt;
use AppConfig::File;

use XgmmlNode;
use XgmmlEdge;

my $distance_method = 0;	#0 ... SSTAT, 1 ... Spacek kmer cosin angle.


my $state;
configure();

# Print the cumulative cover of cluster representatives and get out
my $PRINT_CUMULATIVE_COVER = 0; 

# Put sequence logo links also to PWM nodes, not just clusters
my $ALL_LOGOS = 0;

# Connect cluster nodes to all other nodes, not just representatives
my $ALL_EDGES = 0;

# Stupid thing, if numbers are close to zero and possible negative, it is impossible to set
# Cytoscape visual style
# - just adds a a new attribute with "amplified value" that is big and positive
# did not really help at all
my $AMPLIFY_DISTANCE = 1.0;

# use MAX of distances as an edge instead of MIN (TOMTOM distance is not symmetric
my $MAX_DIST = 0; 

# Threshold for distance
# only edges with smaller or equal distance value are created
my $ET = $state->cutoff_distance();    # reasonable value for TOMTOM E-value might be 0.005

# INPUT FILES
# distance file
my $dist_file = $state->distance_file();

# HT-SELEX family file (mammals)
# - optional
my $MAMMALS = 0;
my $selex_family_file = "";
if ($state->ht_selex_ann_file()) {
    $MAMMALS = 1;
    $selex_family_file = $state->ht_selex_ann_file();
}

# fly SELEX protein name to matrix mappings
my $fly_selex_manifest_file = $state->fly_ht_selex_manifest_file();

# Jaspar file
# - optional
my $JASPAR = 0;
my $jaspar_file = "";
if ($state->jaspar_ann_file()) {
    $JASPAR = 1;
    $jaspar_file = $state->jaspar_ann_file;
}

# Create ILP matrices instead of network 
my $MAKE_DOM_ILP = 0;
$MAKE_DOM_ILP = 1 if ($state->create_dominating_ilp ne "off");
    
# cluster representative file
my $PUT_CLUSTER_CC = 0;
my $cluster_file;
if ($state->cluster_representative_file()) {
    $PUT_CLUSTER_CC = 1;
    $cluster_file = $state->cluster_representative_file();    
}

# Read input files

### construct TF nodes 
my %tf_nodes = ();
my %pwm_nodes = ();
my %tf_to_pwm_edges = ();


my %families = ();
my %uid_check;
my %uid_pfm;

### HT-SELEX  ####
open(FFH, "<$fly_selex_manifest_file") or die "Could not open fly SELEX manifest file $fly_selex_manifest_file: $!";
while (my $line = <FFH>) {
    chomp $line;
    
    next if ($line =~ m/^#/);

    my ($pfm, $name, $type, $family, $spec, $motif_type, $pwm_priority, $comment, undef) = split /\t/, $line;

#    print STDERR "Adding HtSELEX: $name, $pfm\n";
    
    $families{$family}->{$name} = 1;

    # using the pfm file base name as the unique id
#    my $uid = $pfm;
    my $uid = $name."_".$type.$motif_type;
    $uid =~ s/DBD/D/;
    $uid =~ s/Full_length/F/;
    $uid =~ s/Monomer/m/;
    $uid =~ s/Homo_dimer/d/;
    $uid =~ s/Trimer/t/;
    $uid =~ s/Dimer_of_Dimer/D/;
	my $uid_temp = $uid;
	for(my $idx = 2;; $idx++){
		if(!defined($uid_check{$uid_temp})){
			$uid_check{$uid_temp} = 1;
			last;
		}
		$uid_temp = $uid.$idx;
	}
	$uid = $uid_temp;
	$uid_pfm{$pfm} = $uid;

    # Make a node for the tf (if not present yet) 
    # - we bravely assume that names can be used for this purpose  
    my $tf_node;
    if (!defined($tf_nodes{$name})) {
	$tf_node = XgmmlNode->new($name);
	$tf_node->add_attribute("Type", "string", "TF");	
	$tf_node->add_attribute("TFName", "string", $name);
	$tf_node->add_attribute("Family", "string", $family);
	$tf_node->add_attribute("Study", "string", "HT-SELEX");
	
#	$tf_nodes{$name} = $tf_node;
    } else {
#	$tf_node = $tf_nodes{$name};
    }

    # The same for PWM
    my $pwm_node;
    if (!defined($pwm_nodes{$uid})) {
	$pwm_node = XgmmlNode->new($uid);
	$pwm_node->add_attribute("Type", "string", "PWM");
	$pwm_node->add_attribute("Source", "string", "HT-SELEX");
	$pwm_node->add_attribute("Collection", "string", "HT-SELEX");
	$pwm_node->add_attribute("Study", "string", $pfm);
	$pwm_node->add_attribute("Family", "string", $family);
	$pwm_node->add_attribute("Species", "string", $spec);
	
	# DBD or full
	$pwm_node->add_attribute("ProteinType", "string", $type);
	                                                                  
	$pwm_node->add_attribute("ClusterCenter", "integer", 0);

	# Add a link to the sequence logo - fix
	if ($ALL_LOGOS) {
	    my $pwm_file_name = $pfm.".pfm.png";
	    $pwm_node->add_attribute("GraphicsFile", "string", $pwm_file_name);
	}

	$pwm_nodes{$uid} = $pwm_node;
    } else {
	die "Error: Duplicate HT-SELEX annotation";  
    }

    # and link TF node to the PWM node
    my $edge = XgmmlEdge->new("TFspecificity", $tf_node, $pwm_node);
    $edge->add_attribute("Type", "string", "TFtoPWM");
#    $tf_to_pwm_edges{$edge} = $edge;

    # Mark TF as the parent of the PWM
#    $pwm_node->parent($tf_node);
    

}
close(FFH);

print STDERR "FAMILIES\n";
for my $family (sort {scalar(keys %{ $families{$b} }) <=> scalar(keys %{ $families{$a} })} 
		keys %families) {
    print STDERR "\t$family: ".scalar(keys %{ $families{$family} })."\n";
}


# distance matrix file 
my %dist = ();
open(DFH, "<$dist_file") or die "Could not open distance file $dist_file: $!";

# first should be title line
my $line = <DFH>;
chomp $line;
my @col_names = split /\t/, $line;

my $min_val;
my $max_val;
if($distance_method == 0){
	$min_val = 1e9;
	$max_val = -1e9;
}else{
	$min_val = 0.5;
	$max_val = 0.5;
}



while (my $line = <DFH>) {
    chomp $line;

    my @cols = split /\t/, $line;
    my $query = $cols[0];
    # Find the PWM node, if not present should be wrong species
    my $qnode = $pwm_nodes{$uid_pfm{$query}} or next;

    for my $i (1..scalar(@cols)-1) {
	my $target = $uid_pfm{$col_names[$i]};
	my $tnode = $pwm_nodes{$target} or next;
    # Store the distance for later use in edge construction
	my $val = $cols[$i];
	#print STDERR "$query $target $val\n";

	$dist{$uid_pfm{$query}}->{$target} = $val;

	$min_val = $val if ($val < $min_val);
	$max_val = $val if ($val > $max_val);	
    }
}
close(DFH);

# PWM <-> PWM edges according to minimum distance between the two
my %pwm_to_pwm_edges = ();

# alternatively adjacency table for ILP
# - PWM order for adjacency table as distance file can contain more PWMs than are used here
my %pwm_adjt_pos = ();
my $i = 0;
for my $pwm (sort values %pwm_nodes) {
    $pwm_adjt_pos{$pwm} = $i; 
    $i++;
}

# - adjacency table and distance table for ILP
my @adjt = ();
my @distt = ();

my ($min_amp, $max_amp) = (10e9, 0);

for my $source (sort keys %dist) {

    # If no PWM node exists, do not include
    next if (!defined($pwm_nodes{$source}));
    my $snode = $pwm_nodes{$source};

    for my $target (sort keys %{ $dist{$source} }) {

	next if (!defined($pwm_nodes{$target}));
	my $tnode = $pwm_nodes{$target};

	my $dist1 = $dist{$source}->{$target};
	my $dist2 = $dist{$target}->{$source};

	# no self edges
	next if ($source eq $target);

	print STDERR "MISSING ($target, $source)\n" if (!defined($dist2));  

	# NOTE: we only make the edge with the smaller distance if unsymmetric
	# - except if MAX_DIST is on, then the opposites
	#print STDERR "dist1 $dist1, dist2 $dist2\n";
	next if ((!$MAX_DIST && ($dist2 < $dist1 || ($dist2 == $dist1 && $target lt $source))) ||
	         ($MAX_DIST && ($dist2 > $dist1 || ($dist2 == $dist1 && $target lt $source))));

	# Edge distance threshold
	if($distance_method == 0){
		next if ($dist1 > $ET);
	}else{
		next if ($dist1 < $ET);
	}

	#my $diff = $dist1 - $dist2;
	#print "DIST ($source, $target): $dist1 $dist2 \n";

	my $logdist1 = 0;
	$logdist1 = log($dist1) if ($dist1 > 0);

	my $e = XgmmlEdge->new("PWMdistance", $snode, $tnode);
	$e->add_attribute("Type", "string", "PWMtoPWM");
	$e->add_attribute("Distance", "real", $dist1);
	$e->add_attribute("LogDistance", "real", $logdist1);

	if ($AMPLIFY_DISTANCE) {
	    # Trying to create a heuristic similarity that would work in Cytoscape
		my $adist;
		if($distance_method == 0){
		    $adist = exp(100000*($dist1)+1);
		}else{
		    $adist = $dist1*100;
		}
#	    print STDERR "$adist\n";
	    $min_amp = $adist if ($adist < $min_amp);
	    $max_amp = $adist if ($adist > $max_amp);
	    $e->add_attribute("AmplifiedDistance", "real", $adist);
	}

	#print $e->to_string();
	$snode->add_neighbor($tnode);
	$tnode->add_neighbor($snode);
	$pwm_to_pwm_edges{$e} = $e;
	
       	if ($MAKE_DOM_ILP) {
	    
	    my $si = $pwm_adjt_pos{$snode};
	    my $ti = $pwm_adjt_pos{$tnode};
	    $adjt[$si]->[$ti] = 1;
	    $adjt[$ti]->[$si] = 1;
	    $distt[$si]->[$ti] = $dist1;
	    $distt[$ti]->[$si] = $dist1;	
	}
    }
}
print STDERR "min distance: $min_val, max distance: $max_val\n";
print STDERR "amplified for visualization: $min_amp, $max_amp\n";


# Write ILP stuff and get out
if ($MAKE_DOM_ILP) {
    
    my $miss = 0;
    print "set NODES      :=";
    for my $pwm (sort values %pwm_nodes) {
	my $label = $pwm->get_label;
	print "\t\"$label\"";
    }
    print ";\n";
    
    # print adjacency tables with self edges
    print "param AdjTable:";
    for my $pwm (sort values %pwm_nodes) {
	my $label = $pwm->get_label;
	print "\t\"$label\"";
    }
    print ":=\n";
    $i = 0;
    my $li = scalar(keys %pwm_nodes)-1;
    for my $pwm (sort values %pwm_nodes) {
	my $label =  $pwm->get_label;
	print "\"$label\"";
	
	for my $j (0..$li) {
	    if ($i == $j || (defined($adjt[$i]) && defined($adjt[$i]->[$j]) && $adjt[$i]->[$j] == 1)) {
		print "\t1";
	    } else {
		print "\t0";
	    }
	}
	print "\n";
	$i++;
    }
    print ";\n";


    # print distance table without self edges
    if (0) {   # should not be needed for basic dominating set
	print "param DistTable:";
	for my $pwm (sort values %pwm_nodes) {
	    my $label = $pwm->get_label;
	    print "\t\"$label\"";
	}
	print ":=\n";
	$i = 0;
	$li = scalar(keys %pwm_nodes)-1;
	for my $pwm (sort values %pwm_nodes) {
	    my $label =  $pwm->get_label;
	    print "\"$label\"";
	    
	    for my $j (0..$li) {
		if ($i != $j && (defined($adjt[$i]) && defined($adjt[$i]->[$j]) && $adjt[$i]->[$j] == 1)) {
		    print "\t".$distt[$i][$j];
		} else {
		print "\t0";
		}
	    }
	    print "\n";
	    $i++;
	}
	print ";\n";
    }	
    print "end;";
    exit();    
}




# cluster representatives - a simple list of matrix names
my %cluster_nodes = ();
my %pwm_to_cluster_edges = ();
my %pwm_cluster_membership_edges = ();
my %tf_to_cluster_edges = ();
my %tf_cluster_membership_edges = ();

my %tf_clusters = ();

if ($PUT_CLUSTER_CC) {
    my $nae = 0;

    open(CFH, "<$cluster_file") or die "Could not open cluster file $cluster_file: $!";
    while (my $line = <CFH>) {
	chomp $line;
	
	my $cc = $line;
	
	# trim .pfm from the name as TOMTOM does
	$cc =~ s/.pfm\Z//;
	
	my $pwm_node = $pwm_nodes{$cc};
	$pwm_node->add_attribute("ClusterCenter", "integer", 1);
	my $seed = $pwm_node->get_attribute_value("Seed");

	# Make a separate node for the cluster which this node represents
	my $uid = "cluster_".$cc;
	#print STDERR "ADDING CLUSTER $uid\n";
	
	my $clust_node = XgmmlNode->new($uid);
	$clust_node->add_attribute("Type", "string", "CLUSTER");

	# Put seed as TF name so that can be shown as the label
	#$clust_node->add_attribute("TFName", "string", $seed);
	
	# Put Cluster as other attributes as well so that its visual style can be manipulated
	$clust_node->add_attribute("Species", "string", "CLUSTER");
	$clust_node->add_attribute("ProteinType", "string", "CLUSTER");
	
	# For filtering families
	$clust_node->add_attribute("Family", "string", $pwm_node->get_attribute_value("Family"));

	# Add sequence logo file name of the representative PWM to the cluster
	# TODO: hardcoded path, make configurable
	my $pwm_file_name = $cc.".pfm.png";
	$clust_node->add_attribute("GraphicsFile", "string", $pwm_file_name);

	$cluster_nodes{$uid} = $clust_node;

	# Edge between the representative PWM and cluster node
	my $edge = XgmmlEdge->new("ClusterPWM", $pwm_node, $clust_node);
	$edge->add_attribute("Type", "string", "ClusterRepresentative");
	$pwm_to_cluster_edges{$edge} = $edge;	

	# Connect the parent TF node of the PWM to the cluster
	if ($ALL_EDGES) {
	    my $tf_node = $pwm_node->parent;
	    if (!defined($tf_clusters{$tf_node}->{$clust_node})) { 
		my $tf_edge = XgmmlEdge->new("ClusterTF", $tf_node, $clust_node);
		$tf_edge->add_attribute("Type", "string", "TFRepresentative");
		$tf_to_cluster_edges{$tf_edge} = $tf_edge;
		
		$tf_clusters{$tf_node}->{$clust_node} = 1;
	    }
	}
    }
    close(CFH);

    if ($PRINT_CUMULATIVE_COVER) {
	print_cumulative_cover(\%pwm_nodes, \%pwm_to_pwm_edges);
	exit();
    }

    # Color the clusters so that overlapping ones have distinct colors
    color_clusters(\%pwm_nodes, \%pwm_to_pwm_edges); 

    # Connect each PWM node to a cluster via its dominating neighbor
    if ($ALL_EDGES) {
	for my $node (values %pwm_nodes) {
	    next if ($node->get_attribute_value("ClusterCenter") == 1); # dominating itself
	    
	    #print STDERR "node: ".$node->get_label()." number of neighbours ".scalar(@ { $node->get_neighbors })."\n";
	    
	    my $found_dom = 0;		
	    for my $nnode (@{ $node->get_neighbors }) {
		
		if (defined($nnode->get_attribute_value("ClusterCenter"))) {
		    # found a dominating neighbor
		    $found_dom = 1;
		    
		    # Find the cluster node
		    for my $clust_edge (values %pwm_to_cluster_edges) {
			if ($clust_edge->get_source_node->get_label eq $nnode->get_label) {
			    
			    my $clust_node = $clust_edge->get_target_node();
			    
			    # connect PWM to the cluster of its dominating neighbor
			    my $pe = XgmmlEdge->new("PWMCluster", $node, $clust_node);
			    $pe->add_attribute("Type", "string", "ClusterMember");
			    $pwm_cluster_membership_edges{$pe} = $pe;
			    
			    # connect the parent TF of the pwm to the same cluster			
			    my $tf_node = $node->parent();
			    if (!defined($tf_clusters{$tf_node}->{$clust_node})) { 
				my $te = XgmmlEdge->new("TFCluster", $tf_node, $clust_node);
				$te->add_attribute("Type", "string", "TFMember");
				$tf_cluster_membership_edges{$te} = $te;
				$tf_clusters{$tf_node}->{$clust_node} = 1;
			    }
			    
			    last;
			}
		    }
		} 
	    }
	    if ($found_dom == 0) {
		warn "Could not find a dominating neighbor for ".$node->get_label()."\n";
	    }
	}
    }

}


# Write nodes and edges for Cytoscape 
# Note: TOMTOM default distance is not symmetrix, using minimum

# Print graph start, copied from Cytoscape output
if ($JASPAR) {
    print '<graph directed="1" label="HT-SELEX_and_Jaspar_matrices" id="HT-SELEX_and_Jaspar_matrices"  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns="http://www.cs.rpi.edu/XGMML">'."\n";
} else {
    print '<graph directed="1" label="HT-SELEX_matrices" id="HT-SELEX_and_Jaspar_matrices"  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns="http://www.cs.rpi.edu/XGMML">'."\n";
}


# Print nodes
for my $tf_node (values %tf_nodes) {
    print $tf_node->to_string();
}
my $ntf = scalar(values %tf_nodes);
my $npwm = scalar(values %pwm_nodes);
print STDERR "Number of TF nodes   : $ntf\n";
print STDERR "Number of PWM nodes  : $npwm\n";

my $nc = 0;
if ($PUT_CLUSTER_CC) {
    for my $clust_node (values %cluster_nodes) {
	print $clust_node->to_string();
    }   
    $nc = scalar(values %cluster_nodes); 
    print STDERR "Number of CLUSTER nodes  : $nc\n";
}

my $nt = $ntf + $npwm + $nc;
print STDERR "Total number of nodes: $nt\n";

my $miss = 0;
for my $pwm_node (values %pwm_nodes) {
    
    print $pwm_node->to_string();
    
    # Check at the same time that all PWMs have been annotated
    if (!defined($pwm_node->get_attribute_value("Species"))) { 
	warn("No species annotation for PWM node ".$pwm_node->get_label());
	$miss++;
    }
}
print STDERR "Number of PWM nodes missing species annotation: $miss\n";

# Print edges
# TF -> PWM
for my $e (values %tf_to_pwm_edges) {
    print $e->to_string;
}
my $netf = scalar(values %tf_to_pwm_edges);


# PWM<->PWM
my $nepwm = 0;
for my $e (values %pwm_to_pwm_edges) {
    print $e->to_string;
    $nepwm++;
}


print STDERR "Number of TF->PWM edges  : $netf\n";
print STDERR "Number of PWM<->PWM edges: $nepwm\n";

# Edges to cluster nodes
my $nec = 0;
my $necm = 0;
my $tfec = 0;
my $tfecm = 0;
if ($PUT_CLUSTER_CC) {

    # From PWM to the cluster 
    for my $e (values %pwm_to_cluster_edges) {
	print $e->to_string;
    }   
    $nec = scalar(values %pwm_to_cluster_edges);
    print STDERR "Number of representative PWM<->CLUSTER edges: $nec\n";

    for my $e (values %pwm_cluster_membership_edges) { 
	print $e->to_string;
    }
    $necm = scalar(values %pwm_cluster_membership_edges);
    print STDERR "Number of member PWM<->CLUSTER edges: $necm\n";

    # From TF to the cluster 
    for my $e (values %tf_to_cluster_edges) {
	print $e->to_string;
    }   
    $tfec = scalar(values %tf_to_cluster_edges);
    print STDERR "Number of representative TF<->CLUSTER edges: $tfec\n";

    for my $e (values %tf_cluster_membership_edges) {
    	print $e->to_string;
    }
    $tfecm = scalar(values %tf_cluster_membership_edges);
    print STDERR "Number of member TF<->CLUSTER edges: $tfecm\n";
}

my $net = $netf + $nepwm + $nec + $necm + $tfec + $tfecm; 
print STDERR "Total number of edges    : $net\n";
print '</graph>'."\n";



#---------------------------------------------------------------------
# Find such coloring of "clusters" that two "clusters" that share
# a node have a different color
#--------------------------------------------------------------------- 
sub color_clusters {
    my ($pwm_nodes, $pwm_to_pwm_edges) = @_;

    my $max_color = 1;
    my %cum_cover = ();
    
    # Go through all cluster representatives
    # - sort the nodes to largest degree first order
    my $i = 0;
    for my $cnode (sort {$b->degree() <=> $a->degree()} values %{ $pwm_nodes }) {

	next if (!defined($cnode->get_attribute_value("ClusterCenter")) ||
	         !$cnode->get_attribute_value("ClusterCenter"));

	# Check if one of the colors is such that none of the neighbors 
	#(or representative itself has it yet)
	my $found_color = 0;
	for my $ccolor (1..$max_color) {
	    my $used = 0;
	    
	    if (defined($cnode->get_attribute_value("ClusterColor")) && 
		$cnode->get_attribute_value("ClusterColor") == $ccolor) {
		$used = 1;
		#print STDERR "used (1)\n";
		next;
	    }
	    for my $nnode (@{ $cnode->get_neighbors }) {
		if (defined($nnode->get_attribute_value("ClusterColor")) && 
		    $nnode->get_attribute_value("ClusterColor") == $ccolor) {
		    $used = 1;
		    #print STDERR "used (2)\n";
		    last;
		}
	    }
	    if (!$used) {
		$found_color = $ccolor;
		last;
	    }
	}

	# If no such color exists, take a new one
	if (!$found_color) {
	    $max_color++;
	    $found_color = $max_color;
	}
	$i++;
	
	# if requested, print degrees of the representatives
	#if ($DEGREES) {
	    # old simple
	    #print STDERR $cnode->degree()."\t".$cnode->get_label."\n" if ($DEGREES);
	#}

	# color the representative and its neighbors with the chosen one
	# - but separate cluster center
	$cnode->add_attribute("ClusterColor", "integer", $found_color);
	$cnode->add_attribute("ClusterCenterColor", "integer", $found_color);

	for my $nnode (@{ $cnode->get_neighbors }) {

	    # if already has a color, append, otherwise just add the new one
	    # - implicitly assumes that we have less that 10 real colors
	    my $new_color = $found_color;
	    if (defined($nnode->get_attribute_value("ClusterColor"))) {
		$new_color = $nnode->get_attribute_value("ClusterColor").$found_color;
	    } 
	    $nnode->add_attribute("ClusterColor", "integer", $new_color);
	}	
    }
    print STDERR "Colored clusters using $max_color colors\n";

    # Go through all PWM-PWM edges and add color to ones eminating from cluster representatives
    for my $e (values %{ $pwm_to_pwm_edges }) {


	my $s = $e->get_source_node;
	my $t = $e->get_target_node;

	my $s_color = 0;
	my $t_color = 0;
	if (defined($s->get_attribute_value("ClusterCenter")) &&
	    $s->get_attribute_value("ClusterCenter")) {
	    $s_color = $s->get_attribute_value("ClusterCenterColor");
	}
	if (defined($t->get_attribute_value("ClusterCenter")) &&
	    $t->get_attribute_value("ClusterCenter")) {
	    $t_color = $t->get_attribute_value("ClusterCenterColor");
	}

	# if both are cluster centers, default color zero
        if ($s_color && $t_color) {
	     $e->add_attribute("ClusterColor", "integer", 0);
	     next;
	}
		
	# otherwise add the one color, or zero
	if ($s_color) {
	    #print STDERR "Adding edge color (s)\n";
	    $e->add_attribute("ClusterColor", "integer", $s_color);
	} elsif ($t_color) {
	    #print STDERR "Adding edge color (t)\n";
	    $e->add_attribute("ClusterColor", "integer", $t_color);
	} else {
	    $e->add_attribute("ClusterColor", "integer", 0);
	}
    }
}


#--------------------------------------------------------------------
# Calculate and print the cumulative cover of cluster representatives
#---------------------------------------------------------------------
sub print_cumulative_cover {
    my ($pwm_nodes, $pwm_to_pwm_edges) = @_;

    my @covered = ();
    my %uncovered;	
       
    for my $node (values %{ $pwm_nodes }) {
	$uncovered{$node} = 1;
    }

    my %cluster_used = ();
    for my $node (values %{ $pwm_nodes }) {
	if (defined($node->get_attribute_value("ClusterCenter")) &&
	    $node->get_attribute_value("ClusterCenter")) {
	    $cluster_used{$node} = 0;
	}
    }

    # TODO: check which input contains labels, which nodes as keys
    print STDERR "CUMULATIVE COVER\n";

    # Continue until all are covered
    my $i = 0;
    while (scalar(@covered) < scalar(keys %{ $pwm_nodes })) {
	# find the one that covers most of the uncovered ones
	my $best_cover = 0;
	my $best_node;
	my @best_new_covered;
	
	for my $cnode (values %{ $pwm_nodes }) {
	    if (defined($cnode->get_attribute_value("ClusterCenter")) &&
		$cnode->get_attribute_value("ClusterCenter") && !$cluster_used{$cnode}) {

		my $covers = 0;
		$covers++ if ($uncovered{$cnode});

		my @new_covered = ();
		push @new_covered, $cnode->get_label() if ($uncovered{$cnode});

		for my $nnode (@{ $cnode->get_neighbors }) {
		    $covers++ if ($uncovered{$nnode});
		    push @new_covered, $nnode->get_label() if ($uncovered{$nnode});
		}
		if ($covers > $best_cover) {
		    $best_cover = $covers;
		    $best_node = $cnode;
		    @best_new_covered = @new_covered;
		}
	    }
	}
	
	# update - note, even representative can be covered by another one
	$cluster_used{$best_node} = 1;
	if ($uncovered{$best_node}) {
	    $uncovered{$best_node} = 0;
	    push @covered, $best_node;
	}
	for my $nnode (@{ $best_node->get_neighbors() }) {
	    if ($uncovered{$nnode}) {
		$uncovered{$nnode} = 0;
		push @covered, $nnode;
	    }
	}
	
	# print the found covering node and cumulative cover
	$i++;
	print $best_node->get_label, "\t", scalar(@covered), "\t$i\n";
	# print a list of covered 
	#print $best_node->get_label, "\t", join("\t", @best_new_covered), "\n";
    }
}


#-----------------------------
# Read configuration file
#------------------------------
sub configure {

    ## TODO: Add requirements for the parameter types and values

    $state  = AppConfig::State->new();
    my $getopt = AppConfig::Getopt->new($state);
    
    # distance cut-off that determines edges between PWMs
    $state->define("cutoff_distance", {
	                               DEFAULT => 0,
	                               ARGCOUNT => ARGCOUNT_ONE
		                      });
  
    # distance file
    $state->define("distance_file", {
				     ARGCOUNT => ARGCOUNT_ONE
		                    });                                    

    # HT-SELEX family file
    $state->define("ht_selex_ann_file", {
				         ARGCOUNT => ARGCOUNT_ONE
		                        });   

    # file containing fly HT-SELEX protein name to matrix mappings
    $state->define("fly_ht_selex_manifest_file", {
	                                          DEFAULT => "",
				                  ARGCOUNT => ARGCOUNT_ONE
		                                 });  

    # file just listing the B1H matrix file names
    $state->define("b1h_matrix_name_file", {
	                                    DEFAULT => "",
					    ARGCOUNT => ARGCOUNT_ONE
		                           });
    

    # Instead of network output, create dominating set ILP  
    $state->define("create_dominating_ilp", {
	                                     DEFAULT => "off",
				             ARGCOUNT => ARGCOUNT_NONE
		                            });
  
    # Cluster representative file, optional
    $state->define("cluster_representative_file", {
	                                           DEFAULT => "",
				                   ARGCOUNT => ARGCOUNT_ONE
		                                  });   

    # Configuration file
    $state->define("configuration_file", {
	                                  DEFAULT => "",
				          ARGCOUNT => ARGCOUNT_ONE
		                         });     

  

    # Read command line arguments
    $getopt->parse();

    if (defined($state->configuration_file)) {
	printf(STDERR "Reading eading parameter configuration file %s\n", 
	       $state->configuration_file);
    }
    
    # Read configuration file
    my $cfgfile = AppConfig::File->new($state, $state->configuration_file);
}
